# Core
from .core import *

# MachineLearning
from .MachineLearning import *

__version__ = '0.1.3b'
__author__ = 'Zeesain Tsui'
__help__ = """
===============================================
Hello, this is a brief help sheet for oneline.
===============================================

# What is oneline?

oneline is a personal code set for learning and researching, and it provides advanced encapsulation of some libraries,
designing to provide a simpler mode of operation and help people to simplify their codes concisely.

# For example

from oneLine import OneData
data = OneData('test.csv')

Directory or array etc will be converted to OneData, and you can also use OneData as pd.DataFrame.

Have fun!
"""



import json
import shutil
from os.path import isfile, isdir


def ignore(*ignorance):
    """
    A built-in ignore_patterns function.
    """
    return shutil.ignore_patterns(*ignorance)


def copy(source: str, to: str, ignore=None):
    """
    Copy a file or a folder to destination.
    You can use io.ignore to ignore the particular files from copy process.

    For example: io.copy('input', 'new', ignore=io.ignore('*.csv'))
    """
    if isdir(source):
        if ignore:
            shutil.copytree(source, to, ignore=ignore)
        else:
            shutil.copytree(source, to)
    elif isfile(source) and isfile(to):
        shutil.copyfile(source, to)
    else:
        raise TypeError(f"Sorry, the source is { 'directory' if isdir(source) else 'file' } but destination is { 'directory' if isdir(to) else 'file' }")


def delete(source):
    """
    Delete the file or folder.
    """
    shutil.rmtree(source)


def move(source: str = None, to: str = None):
    """
    Move file or folder.
    """
    shutil.move(source, to)


def compress(source: str = None, to: str = None, _format: str = None):
    """
    Compress files to a file.
    """
    shutil.make_archive(to, _format, root_dir=source)


def extract(source: str = None, to: str = None):
    """
    Extract a compress file.
    """
    _format = source.split('.')[-1]
    shutil.unpack_archive(filename=source, extract_dir=to, format=_format)


def read(path: str, mode: str = 'r+', readlines: bool = False, encoding: str = 'utf-8'):
    """
    That's a simple and fast function for reading a file.
    It will return a string that contains all the content when readlines = False.
    And will return a list if readlines = True.

    WARNING: This is for only small size of files, which means that the bigger size will lead to a memory exception.
    """
    with open(path, mode, encoding=encoding) as f:
        if readlines:
            return f.readlines()
        else:
            return f.read()


def write(content, path, mode: str = 'w+'):
    """
    Write string or list to file.

    If the content is a list, it will be written with writelines mode.
    And also the write mode will be used if the content is string.
    """
    with open(path, mode) as f:
        if isinstance(content, list):
            f.writelines(content)
        elif isinstance(content, str):
            f.write(content)


def load_json(path: str):
    """
    Load the json file, which will return a dictionary.
    """
    with open(path, 'r', encoding='utf-8') as f:
        return json.load(f)


def save_json(_dict: dict, path: str, indent=4, ensure_ascii=False):
    """
    Save a dictionary to json file
    """
    with open(path, 'w', encoding='utf-8') as f:
        f.write(json.dumps(_dict, indent=indent, ensure_ascii=ensure_ascii))



import sys
import traceback
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d


class Plot:
    """
    This is a plot module for data visualization.
    """

    def _raise_plot_value_error(self, s: str):
        raise ValueError(f'The parameter {s} missed.')

    def __init__(self, *args):
        self = pd.DataFrame()

    def line_plot(self, x: str = None,
                  y: list = None,
                  figsize: list = None,
                  title: str = None,
                  xlabel: str = None,
                  ylabel: str = None,
                  smooth: bool = False,
                  insert_num: int = 50,
                  label_loc: str = 'upper left',
                  show: bool = True):
        """
        It's a fast plot function to generate graph rapidly.
        :param x: the x
        :param y: the y
        :param figsize: The size of figure.
        :param title: the title of plot
        :param xlabel: label of x
        :param ylabel: label of y
        :param smooth: Set it True if the curve smoothing needed.
        :param insert_num: define the number for smooth function.
        :param label_loc: The location of the labels in plot.
        :param show: plt.show will run if true
        """
        sns.set()
        if figsize:
            plt.figure(figsize=figsize)
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        if x:
            x = self[x]
        else:
            x = list(self.index)
        if y:
            for name in y:
                y_val = self[name]
                if smooth:
                    x_new = np.linspace(min(x), max(x), len(x) * insert_num)
                    y_smooth = interp1d(x, y_val, kind='cubic')
                    plt.plot(x_new, y_smooth(x_new), label=name)
                else:
                    plt.plot(x, y_val, label=name)
                plt.legend(loc=label_loc)
        else:
            for name in list(self.columns):
                y_val = self[name]
                if smooth:
                    x_new = np.linspace(min(x), max(x), len(x) * insert_num)
                    y_smooth = interp1d(x, y_val, kind='cubic')
                    plt.plot(x_new, y_smooth(x_new), label=name)
                else:
                    plt.plot(x, y_val, label=name)
                plt.legend(loc='upper left')
        if show:
            plt.show()

    def count_plot(self, x: str = None,
                   hue: str = None,
                   figsize: list = None,
                   show: bool = True):
        """
        Generate the count graph
        :param x: The x.
        :param hue: The hue.
        :param figsize: The size of figure.
        :param show: plt.show will run if true
        """
        try:
            if not x:
                self._raise_plot_value_error('x')
        except Exception:
            traceback.print_exc(limit=1, file=sys.stdout)
            quit()

        sns.set()
        if figsize:
            plt.figure(figsize=figsize)
        sns.countplot(x=x, hue=hue, data=self)
        if show:
            plt.show()

    def corr_plot(self, parameters: list = None,
                  figsize: list = None,
                  annot: bool = True,
                  show: bool = True):
        """
        Generate the correction graph
        :param parameters: The parameters selected.
        :param figsize: The size of figure.
        :param annot: Display the annotation or not.
        :param show: plt.show will run if true
        """
        sns.set()
        if figsize:
            plt.figure(figsize=figsize)
        if parameters:
            sns.heatmap(self[parameters].corr(), annot=annot)
        else:
            sns.heatmap(self.corr(), annot=annot)
        if show:
            plt.show()

    def comparing_variables(self, variable1: str = None,
                            variable2: str = None,
                            show: bool = True):
        try:
            if not variable1 or not variable2:
                self._raise_plot_value_error('variable1 and variable2')
            elif self[variable1].dtype == 'object' or self[variable2].dtype == 'object':
                raise TypeError('The type of parameters should be int or float, rather than object.')
        except Exception:
            traceback.print_exc(limit=1, file=sys.stdout)
            quit()

        print(self[[variable1, variable2]][self[variable2].isnull() == False].groupby([variable1],
                                                                                                as_index=False).mean().sort_values(
            by=variable2, ascending=False))
        sns.FacetGrid(self, col=variable2).map(sns.distplot, variable1)
        if show:
            plt.show()


def line_plot(x: list = None,
              y: dict or list = None,
              figsize: list = None,
              title: str = None,
              xlabel: str = None,
              ylabel: str = None,
              smooth: bool = False,
              insert_num: int = 50,
              show: bool = True):
    """
    That's a isolate line_plot for faster usage.
    """
    sns.set()
    if figsize:
        plt.figure(figsize=figsize)
    plt.title(title)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)

    if isinstance(y, dict):
        for (name, i) in y.items():
            if smooth:
                x_new = np.linspace(min(x), max(x), len(x) * insert_num)
                y_smooth = interp1d(x, i, kind='cubic')
                if name:
                    plt.plot(x_new, y_smooth(x_new), label=name)
                else:
                    plt.plot(x_new, y_smooth(x_new))
            elif name:
                plt.plot(x, i, label=name)
            else:
                plt.plot(x, i)
            plt.legend(loc='upper left')
    elif isinstance(y, list):
        if smooth:
            x_new = np.linspace(min(x), max(x), len(x) * insert_num)
            y_smooth = interp1d(x, y, kind='cubic')
            plt.plot(x_new, y_smooth(x_new))
        else:
            plt.plot(x, y)
    if show:
        plt.show()


def bar_plot(x: list = None,
             y: list = None,
             figsize: list = None,
             title: str = None,
             xlabel: str = None,
             ylabel: str = None,
             show: bool = True):
    """
    That's a isolate bar_plot for faster usage.
    """
    sns.set()
    if figsize:
        plt.figure(figsize=figsize)
    plt.title(title)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)

    sns.barplot(x=x, y=y)
    if show:
        plt.show()



from sklearn import svm
from sklearn import tree
from sklearn.cluster import KMeans
from sklearn.naive_bayes import GaussianNB
from sklearn.linear_model import LinearRegression
from sklearn.neighbors import KNeighborsClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.preprocessing import PolynomialFeatures


class MachineLearning:
    """
    That's the machine learning module for analysis using, and it's based on the scikit-learn module, which contains
    10 necessary ML methods.

    For example:
    a = MachineLearning(x=x, y=y)  # init with x, y data
    a.linear_reqression()  # train with x, y data
    a.predict(x_test)  # predict with the x_test data using trained model above
    """
    def __init__(self, x: list = None, y: list = None):
        """
        Initialze the dataset for modeling
        :param x: the x data
        :param y: the y data
        """
        self.x = x
        self.y = y
        self.model = None

    def set_value(self, x: list = None, y: list = None):
        """
        Reset value method
        :param x: the new x data
        :param y: the new y data
        """
        self.x = x
        self.y = y

    def linear_regression(self, degree: int = 1, include_bias: bool = False, info: bool = False):
        """
        Linear Regression Method
        :param degree: the degree
        :param include_bias: include bias or not
        :param info: show the info and parameters of the model
        """
        x_train = self.x
        y_train = self.y

        self.model = LinearRegression()
        if degree > 1:
            pf = PolynomialFeatures(degree=degree, include_bias=include_bias)
            x_train = pf.fit_transform(x_train)
        self.model.fit(x_train, y_train)
        self.model.score(x_train, y_train)
        if info:
            print('Coefficient: \n', self.model.coef_)
            print('Intercept: \n', self.model.intercept_)

    def logistic_regression(self, info: bool = False):
        """
        Logistic Regression Method
        :param info: show the info and parameters of the model
        """
        x_train = self.x
        y_train = self.y

        self.model = LogisticRegression()
        self.model.fit(x_train, y_train)
        self.model.score(x_train, y_train)
        if info:
            print('Coefficient: \n', self.model.coef_)
            print('Intercept: \n', self.model.intercept_)

    def decision_tree(self):
        """
        Decision Tree Method
        """
        x_train = self.x
        y_train = self.y

        self.model = tree.DecisionTreeClassifier(criterion='gini')
        self.model.fit(x_train, y_train)
        self.model.score(x_train, y_train)

    def svm(self):
        """
        Support Vector Machine Method
        """
        x_train = self.x
        y_train = self.y

        self.model = svm.svc()
        self.model.fit(x_train, y_train)
        self.model.score(x_train, y_train)

    def naive_bayes(self):
        """
        Naive Bayes Method
        """
        x_train = self.x
        y_train = self.y

        self.model = GaussianNB()
        self.model.fit(x_train, y_train)

    def knn(self, n_neighbors=6):
        """
        KNN methods
        :param n_neighbors: the number of n-neighbors
        """
        x_train = self.x
        y_train = self.y

        self.model = KNeighborsClassifier(n_neighbors=n_neighbors)
        self.model.fit(x_train, y_train)

    def kmeans(self, n_clusters=6, random_state=0):
        """
        K-Means Method
        :param n_clusters: the number of clusters
        :param random_state: the number of random state
        """
        x_train = self.x

        self.model = KMeans(n_clusters=n_clusters, random_state=random_state)
        self.model.fit(x_train)

    def random_forest(self):
        """
        Random Forest Method
        """
        x_train = self.x
        y_train = self.y

        self.model = RandomForestClassifier()
        self.model.fit(x_train, y_train)

    def predict(self, x_test: list):
        """
        Predict method for prediction
        :param x_test: the data for prediction
        """
        return self.model.predict(x_test)



import numpy as np
from abc import ABC
import pandas as pd
from pandas import DataFrame
from pandas.api.types import is_categorical_dtype
from pandas.api.types import is_datetime64_any_dtype as is_datetime
from pandas._libs import lib
from pandas.core.dtypes.common import (
    is_hashable,
    is_integer,
    is_iterator,
    is_list_like,
)
from pandas.core.dtypes.generic import ABCMultiIndex
from pandas.core import common as com
from pandas.core.indexing import convert_to_index_sliceable

from .plot import Plot
from .oneseries import OneSeries


class OneData(DataFrame, ABC, Plot):
    """
    That's the initial of the OneData class.
    Acceptable type:
        1. OneData
        2. DataFrame
        3. List
        4. Numpy Array
        5. String (the string should be a address of particular file, which can be automatically input based on format
                   by OneData)
    OneData is a advanced class of Pandas.DataFrame with more methods and all the features of DataFrame inherited.
    """

    def _raise_value_error(self):
        """
        The exception of ValueError.
        :return: ValueError
        """
        raise ValueError('Input format error, please in put a valid dataset that satisfied OneData.')

    def __init__(self, *args, index=None, columns=None, dtype=None, copy: bool = False):
        """
        Input data or address string, and convert it to OneData format.
        Now the input args supported:
            1. str: a string will be recognized as the location of a file, and input with the format correctly
            2. DataFrame & OneData
            3. Dictionary
            4. Numpy Array
            ...
        OneData inherits all the features of DataFrame.
        """
        data = {}
        if not args:
            pass
        elif isinstance(args[0], str):
            file_form = args[0].split('.')[-1]
            if file_form == 'csv':
                data = pd.read_csv(args[0])
            elif file_form == 'xls' or file_form == 'xlsx':
                data = pd.read_excel(args[0])
            elif file_form == 'json':
                data = pd.read_json(args[0])
            elif file_form == 'pkl':
                data = pd.read_pickle(args[0])
            elif file_form == 'hdf':
                data = pd.read_hdf(args[0])
            else:
                self._raise_value_error()
        else:
            data = args[0]
        try:
            super().__init__(data=data, index=index, columns=columns, dtype=dtype, copy=copy)
        except (ValueError, TypeError):
            self._raise_value_error()

    def __getitem__(self, key):
        """
        The rebuilt method of __getitem__, which will redirect the DataFrame and Series to OneData and OneSeries.
        """
        key = lib.item_from_zerodim(key)
        key = com.apply_if_callable(key, self)

        if is_hashable(key):
            if self.columns.is_unique and key in self.columns:
                if self.columns.nlevels > 1:
                    return OneSeries(self._getitem_multilevel(key))
                return OneSeries(self._get_item_cache(key))

        indexer = convert_to_index_sliceable(self, key)
        if indexer is not None:
            return OneData(self._slice(indexer, axis=0))

        if isinstance(key, DataFrame):
            return self.where(key)

        if com.is_bool_indexer(key):
            return OneData(self._getitem_bool_array(key))

        is_single_key = isinstance(key, tuple) or not is_list_like(key)

        if is_single_key:
            if self.columns.nlevels > 1:
                return self._getitem_multilevel(key)
            indexer = self.columns.get_loc(key)
            if is_integer(indexer):
                indexer = [indexer]
        else:
            if is_iterator(key):
                key = list(key)
            indexer = self.loc._get_listlike_indexer(key, axis=1, raise_missing=True)[1]

        if getattr(indexer, "dtype", None) == bool:
            indexer = np.where(indexer)[0]

        data = self._take_with_is_copy(indexer, axis=1)

        if is_single_key:
            if data.shape[1] == 1 and not isinstance(self.columns, ABCMultiIndex):
                data = data[key]

        return data

    # ---------------------------------------------------------------- #

    def show(self, info: bool = False):
        """
        Display the info of data.
        :param info: stay True if a display of information is required
        :return None
        """
        if info:
            self.info()
            print('\n - Shape: {}\n - Index:{}\n - Memory usage: {:.3f} MB\n'
                  .format(self.shape, list(self.columns), self.memory_usage().sum() / 1024 ** 2))
        print(self)

    def make_dataset(self, train: float = 0.0,
                     holdout: float = 0.0,
                     save: bool = False,
                     filepath: str = None):
        """
        Create dataset function.
        The proportion of the train data and hold-out data should be specified.
        :param train: the proportion of train data
        :param holdout: the proportion of holdout data
        :param save: set True if saving dataset is required
        :param filepath: the path of dataset
        """
        index_num = int(self.shape[0] * train)
        train_data = self.iloc[:index_num, :]
        holdout_data = None

        if holdout:
            holdout_num = int(self.shape[0] * holdout)
            test_data = self.iloc[index_num:index_num + holdout_num, :]
            holdout_data = self.iloc[index_num + holdout_num:, :]
        else:
            test_data = self.iloc[index_num:, :]
        if save:
            train_data.to_csv(path_or_buf=filepath.split('.')[0] + '_train.csv', index=False)
            test_data.to_csv(path_or_buf=filepath.split('.')[0] + '_test.csv', index=False)
            if holdout:
                holdout_data.to_csv(path_or_buf=filepath.split('.')[0] + '_holdout.csv', index=False)
        elif holdout:
            return OneData(train_data), OneData(holdout_data), OneData(test_data)
        else:
            return OneData(train_data), OneData(test_data)

    def summary(self, info: bool = True):
        """
        Return a summary of the whole dataset.
        the stats from scipy is used to calculate the Entropy.
        :param info: stay True if a display of information is required
        """
        from scipy import stats

        pd.set_option('display.max_columns', None)
        print(f"Dataset Shape: {self.shape}")
        summ = pd.DataFrame(self.dtypes, columns=['dtypes'])
        summ = summ.reset_index()
        summ['Name'] = summ['index']
        summ = summ[['Name', 'dtypes']]
        summ['Missing'] = self.isnull().sum().values
        summ['Uniques'] = self.nunique().values
        summ['First Value'] = self.loc[self.index[0]].values
        summ['Second Value'] = self.loc[self.index[1]].values
        summ['Third Value'] = self.loc[self.index[2]].values

        for name in summ['Name'].value_counts().index:
            summ.loc[summ['Name'] == name, 'Entropy'] = round(
                stats.entropy(self[name].value_counts(normalize=True), base=2), 2)
        if info:
            print(summ)
        return summ

    def fill_na(self, method: str = 'mode'):
        """
        Fill the NaN values.
        :param method: the way to fill the NaN values, which contains mode and nan methods
        """
        data = self
        if method == 'mode':
            for key, value in data.isnull().sum().items():
                if value:
                    data[key].fillna(self[key].mode()[0], inplace=True)
        elif method == 'nan':
            data = data.fillna(np.nan)
        return OneData(data)

    def remove(self, column: list = None, row: list = None):
        """
        The advanced function of DataFrame.drop, you can input a list of index to drop them.
        :param column: the list of column you want to remove
        :param row: the list of row you want to remove
        """
        data = self
        if column is None:
            column = []
        if column:
            data = data.drop(column, axis=1)
        if row:
            for n in row:
                data = data.drop(n)
        return OneData(data)

    def add_var(self, exist, new, mapper=None):
        """
        Add a new variable based on the calculating of exist variable.
        :param exist: the exist variable column
        :param new: the new variable column
        :param mapper: the mapper applied to the exist variable
        """
        data = self
        if mapper:
            data[new] = list(map(mapper, data[exist]))
        else:
            data[new] = data[exist]
        return data

    def means(self, variable: str = None, hue: str = None):
        """
        An advanced method of mean, which can calculate means with hue
        :param variable: the variable to calculate means
        :param hue: the hue for calculation
        """
        if hue:
            return self[[hue, variable]][self[variable].isnull() == False].groupby([hue],
                                                                                   as_index=False).mean().sort_values(
                by=variable, ascending=False)
        elif variable:
            return self.mean()[variable]
        else:
            return self.mean()

    def reduce_mem_usage(self, use_float16: bool = False, info: bool = True):
        """
        Automatically distinguish the type of one single data and reset a suitable type.
        :param use_float16: use float16 or not
        :param info: stay True if a display of information is required
        """
        start_mem = self.memory_usage().sum() / 1024 ** 2
        if info:
            print("Memory usage of DataFrame is {:.3f} MB".format(start_mem))

        for col in self.columns:
            if is_datetime(self[col]) or is_categorical_dtype(self[col]):
                continue
            col_type = self[col].dtype

            if col_type != object:
                c_min = self[col].min()
                c_max = self[col].max()
                if str(col_type)[:3] == "int":
                    if c_min > np.iinfo(np.int8).min and c_max < np.iinfo(np.int8).max:
                        self[col] = self[col].astype(np.int8)
                    elif c_min > np.iinfo(np.int16).min and c_max < np.iinfo(np.int16).max:
                        self[col] = self[col].astype(np.int16)
                    elif c_min > np.iinfo(np.int32).min and c_max < np.iinfo(np.int32).max:
                        self[col] = self[col].astype(np.int32)
                    elif c_min > np.iinfo(np.int64).min and c_max < np.iinfo(np.int64).max:
                        self[col] = self[col].astype(np.int64)
                else:
                    if use_float16 and c_min > np.finfo(np.float16).min and c_max < np.finfo(np.float16).max:
                        self[col] = self[col].astype(np.float16)
                    elif c_min > np.finfo(np.float32).min and c_max < np.finfo(np.float32).max:
                        self[col] = self[col].astype(np.float32)
                    else:
                        self[col] = self[col].astype(np.float64)
            else:
                self[col] = self[col].astype("category")

        if info:
            end_mem = self.memory_usage().sum() / 1024 ** 2
            print("Memory usage after optimization is: {:.3f} MB".format(end_mem))
            print("Decreased by {:.1f}%".format(100 * (start_mem - end_mem) / start_mem))

        return OneData(self)

    def iter(self, index=True, name="OneData"):
        """
        Iteration methods for fast using
        :param index: If True, return the index as the first element of the tuple.
        :param name: The name of the returned named tuples or None to return regular.
        """
        return self.itertuples(index=index, name=name)

    def select_raw(self, indices: dict, reset_index: bool = False):
        """
        Select raw with particular indices.

        For example:
        a = a.select_raw({'Gender': 'Male'})  # it will returns the data that matched variable'Gender' == values'Male'
        a = a.select_raw({'Gender': 'Male', 'Age': 20})  # it can also use for multiple matching
        """
        df = self
        for variable, value in indices.items():
            df = df[(df[variable] == value)]
        if reset_index:
            df = df.reset_index()
        return OneData(df)



import numpy as np
import seaborn as sns
from pandas import Series
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d


class OneSeries(Series):
    def __init__(self, data):
        super().__init__(data=data)

    @property
    def _one_data(self):
        from .onedata import OneData
        return OneData

    def summary(self, info: bool = True):
        """
        Return a summary of the whole OneSeries dataset.
        the stats from scipy is used to calculate the Entropy.
        :param info: stay True if a display of information is required
        """
        from scipy import stats

        sum_info = {}
        length = self.shape[0]
        if str(self.dtype)[:3] == "int":
            sum_info['Unique'] = len(self.unique())
            sum_info['Unique(%)'] = sum_info['Unique'] / length
            sum_info['Missing'] = self.isnull().sum()
            sum_info['Missing(%)'] = sum_info['Missing'] / length
            sum_info['Means'] = self.sum() / length
            sum_info['Minimum'] = self.min()
            sum_info['Maximum'] = self.max()
            sum_info['Zeros'] = (self == 0).sum()
            sum_info['Zeros(%)'] = sum_info['Zeros'] / length
            sum_info['Entropy'] = round(
                stats.entropy(self.value_counts(normalize=True), base=2), 2)
            sum_info['Memory Size(KB)'] = self.memory_usage() / 1024

            if info:
                print('Unique: {}({:.2f}%)'.format(sum_info['Unique'], sum_info['Unique(%)']))
                print('Missing: {}({:.2f}%)'.format(sum_info['Missing'], sum_info['Missing(%)']))
                print('Zeros: {}({:.2f}%)'.format(sum_info['Zeros'], sum_info['Zeros(%)']))
                print('Means: {:.2f}'.format(sum_info['Means']))
                print('Minimum: {}'.format(sum_info['Minimum']))
                print('Maximum: {}'.format(sum_info['Maximum']))
                print('Entropy: {}'.format(sum_info['Entropy']))
                print('Memory Size: {:.1f}KB'.format(sum_info['Memory Size(KB)']))

        elif str(self.dtype) == "object":
            sum_info['Unique'] = len(self.unique())
            sum_info['Unique(%)'] = sum_info['Unique'] / length
            sum_info['Missing'] = self.isnull().sum()
            sum_info['Missing(%)'] = sum_info['Missing'] / length
            sum_info['Entropy'] = round(
                stats.entropy(self.value_counts(normalize=True), base=2), 2)
            sum_info['Memory Size(KB)'] = self.memory_usage() / 1024

            if info:
                print('Unique: {}({:.2f}%)'.format(sum_info['Unique'], sum_info['Unique(%)']))
                print('Missing: {}({:.2f}%)'.format(sum_info['Missing'], sum_info['Missing(%)']))
                print('Entropy: {}'.format(sum_info['Entropy']))
                print('Memory Size: {:.2f}KB'.format(sum_info['Memory Size(KB)']))

        return self._one_data(sum_info, index=[0])

    def count_plot(self,
                   figsize: list = None,
                   title: str = None,
                   xlabel: str = None,
                   ylabel: str = None):
        """
        Generate the count graph.
        """
        sns.set()
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        if figsize:
            plt.figure(figsize=figsize)
        sns.countplot(x=self)
        plt.show()

    def line_plot(self,
                  figsize: list = None,
                  title: str = None,
                  xlabel: str = None,
                  ylabel: str = None,
                  smooth: bool = False,
                  insert_num: int = 50,
                  show: bool = True):
        """
        It's a fast plot function to generate graph rapidly.
        """
        sns.set()
        if figsize:
            plt.figure(figsize=figsize)
        plt.title(title)
        plt.xlabel(xlabel)
        plt.ylabel(ylabel)

        x_val = list(range(len(self)))

        if smooth:
            x_new = np.linspace(min(x_val), max(x_val), len(x_val) * insert_num)
            y_smooth = interp1d(x_val, self, kind='cubic')
            plt.plot(y_smooth(x_new), x_new)
        else:
            plt.plot(x_val, self)
        if show:
            plt.show()

    def to_frame(self, name=None):
        if name is None:
            df = self._onedata(self)
        else:
            df = self._onedata({name: self})
        return df
